#
# Another take at SGML->HTML conversion....
#

needExtension ITCL

### Misc. utilities:

substitution htmlEscape {
	{<} {&lt;}
	{>} {&gt;}
	{&} {&amp;}
}

proc makeURL {base path {anchor ""}} {
    regsub {/[^/]*$} $base {/} url
    append url $path
    if {$anchor != ""} {
	append url "#$anchor"
    }
    return $url
}

proc identity {s} { return $s }

### HTML Node base class:

itcl_class HTMLNode {
    common doctype "-//IETF//DTD HTML//EN"
    common destDir "./"

    protected fp stdout
    protected links {}
    public name
    public title ""

    constructor {config} { }

    method output {text} {
	puts $fp $text nonewline
    }
    method cdata {text} {
	output [htmlEscape $text]
    }
    method nl {} {
	output "\n"
    }
    method startTag {gi args} {
	output "<$gi"
	if {[llength $args] == 1} { set args [lindex $args 0] }
	while {[llength $args] > 0} {
	    set attname [lindex $args 0]
	    set attval  [lindex $args 1]
	    set args [lreplace $args 0 1]

	    # handle minimization:
	    if {$attname == $attval} {
		set attspec $attname
	    } elseif {[regexp {^[a-zA-Z][a-zA-Z0-9]*$} $attval]} {
		set attspec "$attname=$attval"
	    } else {
		set attspec "$attname=\"$attval\""
	    }

	    output " $attspec"
	}
	output ">"
    }
    method endTag {gi} {
	output "</$gi>"
    }

### File manipulation:
    method writeToFile {filename} {
	set fp [open $destDir/$filename w]
    }
    proc setDestDir {dirname} {
	set destdir $dirname
    }
###

    method addLink {args} {
	lappend links $args
    }

    method start {} {
	global COST

	output "<!DOCTYPE HTML PUBLIC \"$doctype\">" ; nl
	output \
    "<!-- This file was automatically generated.  DO NOT EDIT BY HAND. -->"; nl
	startTag HTML; nl
	startTag HEAD ; nl
	startTag TITLE
	cdata $title
	endTag TITLE ; nl

	startTag META NAME "Creator" CONTENT "Cost $COST(VERSION)"; nl

	foreach link $links {
	    startTag LINK $link ; nl
	}

	endTag HEAD; nl
	startTag BODY;
    }

    method finish {} {
	endTag BODY
	endTag HTML
    }
}

#HTMLNode dummyNode;
global curnode
set curnode dummyNode;

proc output {data} 		{ global curnode; $curnode output $data }
proc cdata {data} 		{ global curnode; $curnode cdata $data }
proc nl {} 	 		{ global curnode; $curnode nl }
proc startTag {gi args} 	{ global curnode; $curnode startTag $gi $args }
proc endTag {gi} 		{ global curnode; $curnode endTag $gi }

proc includeFile {filename} {
    if [catch {set fp [open $filename r]}] {
	warning "Cannot open $filename" 
	return;
    }
    output [htmlEscape [read $fp]]
    close $fp
}
