# Spell.tcl
# CoST wrapper around 'ispell'

needExtension ITCL

itcl_class SpellChecker {
    inherit EventHandler;

    public dictfile ""
    public suppress ""; 	# list of elements not to spellcheck
    public tmpfile "/tmp/costspell.tmp"; 

    protected ispellpipe; 	# pipe to 'ispell' process
    protected suppressing 0;	# flag: currently suppressing output?
    protected wordlist "";	# local dictionary

    constructor {config} {
	# make sure suppress GI list is all uppercase
	set suppress [string toupper $suppress]
    }

    method suppress? {} {
	# suppress checking for current element?
	return [expr [lsearch $suppress [query gi]] != -1]
    }

    # The START and END tag handlers just set the 'suppressing' flag,
    # and make sure there's whitespace between element boundaries.
    method START {} {
	if [suppress?] { incr suppressing }
    }
    method END {} {
	if [suppress?] { incr suppressing -1 }
	puts $ispellpipe ""
    }

    # Feed character data to 'ispell':
    method CDATA {} {
	if !$suppressing { puts $ispellpipe [content] }
    }

    method PI {} {
	# Is this a <?spelling ...> instruction?
	if {[lindex [query content] 0] == "spelling"} {
	    # Yep; add to local dictionary:
	    append wordlist " [lrange [query content] 1 end]"
	}
    }

    method begin {} {
	set cmd "ispell -l"
	if {$dictfile != ""} { append cmd " -p $dictfile" }
	set ispellpipe [open "|$cmd | sort | uniq > $tmpfile" w]
	set suppressing 0;
    }

    method end {} {
	close $ispellpipe
	# Read words back from temporary file
	set fp [open $tmpfile r] 
	while {[gets $fp word] > 0} {
	    # see if it's in local dictionary:
	    if {[lsearch $wordlist $word] == -1} {
		# nope; report it:
		puts stdout $word
	    }
	}
	close $fp
    }
}

