# conversion to HTML, take 3

require costdoc.spec
require HTML.tcl

HTMLNode :: setDestDir ./fmt

global includeGraphics; set includeGraphics 0
global includeEdnotes; set includeEdnotes 0
global secnumdepth; set secnumdepth 5;

specification tohtml {
    {sd} {
	content { withNode subtree element BODY doNode }
    }
    {element SECT} {
	content	{
	    global secnumdepth

	    set headingTag "H[expr [query propval seclevel] + 1]"
	    nl
	    startTag $headingTag
	    startTag A NAME [query propval reflabel]
	    if {[query propval seclevel] <= $secnumdepth} {
		output "[query propval refnumber] "
	    }
	    withNode child element TITLE doChildren
	    endTag A
	    endTag $headingTag
	    nl

	    withNode child element SUBSECS {
		startTag HR; nl
		minitoc 1
		startTag HR; nl
	    }

	    doChildren
	}
    }

    {element TITLE in SECT} { content "" }

    {element SUBSECS} { content { doChildren } }

    {element BODY} {
	content	{
	    nl
	    startTag H1
	    withNode doctree element HEAD subtree element TITLE  doChildren
	    endTag H1
	    nl

	    startTag P;
	    withNode doctree element HEAD subtree element AUTHOR  {
		doChildren; startTag BR
	    }

	    startTag I;
	    output "Last updated: [exec date]";
	    endTag I

	    startTag HR
	    minitoc -1;
	    startTag HR

	    doChildren
	}
    }

    {element TITLE}  { result H3 }

    {elements "DESC P"} {
	content {
		startTag P
		doChildren
	}
    }

    {element EDNOTE} {
        content {
	    global includeEdnotes
	    if $includeEdnotes {
		startTag P; startTag i;
		output {[[ };
		doChildren
		output { ]]};
		endTag i;
	    }
	}
    }
    {element NOTE} {
	content {
	    nl
	    startTag P
	    output "NOTE -- "
	    doChildren
	}
    }

    {element GRAPHIC hasatt SOURCE} {
	content {
	    global includeGraphics
	    startTag BR
	    withNode entity [query attval SOURCE] {
		if $includeGraphics {
		    startTag A HREF [query sysid]
		    $spec do content
		    endTag A
		} else {
		    startTag P
		    startTag A HREF [query sysid]
		    output "Picture..."
		    endTag A
		    endTag P
		}
	    }
	    startTag BR
	}
    }
    {dataent withdcn GIF} {
	content {
	    startTag IMG SRC [q sysid] ALT [q sysid]
	}
    }
    {dataent withdcn ASCII} {
	content {
	    includeFile [q sysid]
	}
    }
    {dataent} {
	content {
	    warning "Cannot handle data content notation [q dcn]"
	}
    }

    {sdata} 	{ content { output [content] } }
    {cdata}  	{ content { output [htmlEscape [content]] } }
    {re}  	{ content { nl } }

    {elements "CLASS CMD SUBCMD QC EVENT NODETYPE"} {
	 result B
    }
    {element ENV}		{ result TT  prefix {$} }
    {elements "ARG OPTARG"}	{ result I }
    {element OPTION}		{ result B }

    {element KEY}		{ prefix "&lt;" suffix "&gt;" }
    {elements "MENU MENUITEM BTN"}	{ prefix {[} suffix {]} }

    {elements "SAMP LIT"}	{ result SAMP }

    {element VAR}		{ result VAR }
    {element KW}		{ result B }
    {element FUNC}		{ result TT }

    {elements "ELEM ATTR"}	{ result TT }
    {element TAG} {
	result TT
	prefix "&lt;"
	suffix "&gt;"
    }

    {element DFN} 		{ result EM }
    {element DPH} 		{ result EM }
    {element EMPH} 		{ result EM }

    {element H} 	{ result H2 }

    {elements "EXAMPLE LISTING" hasatt SOURCE} {
	content {
	    nl; startTag PRE; nl
	    includeFile [q entity [q attval SOURCE] sysid]
	    endTag PRE; nl;
	}
    }
    {element SYNOPSIS}  { result PRE  prefix "\n" suffix "\n"  }
    {element EXAMPLE} 	{ result PRE  prefix "\n" suffix "\n"  }
    {element LISTING} 	{ result PRE  prefix "\n" suffix "\n"  }
    {element SYNTAX} 	{ result PRE  prefix "\n" suffix "\n"  }

    {element LIST}	{ result UL }
    {element ENUM}	{ result OL }
    {element IT}	{ result LI }
    {element DLIST}	{ result DL }
    {element DT}	{ result DT }
    {element DD}	{ result DD }

    {element REFERENCES} { result DL }
    {element REFENTRY} {
	content {
		startTag DT
		output {[}
		if [query? hasprop refkey] {
		    output [query propval refkey]
		} else {
		    output "???"
		}
		output {]}
		nl;
		startTag DD
		doChildren
	}
    }
    {element XREF} {
	content {
	    output {[}
	    withNode doctree withattval ID [query attval REFID] {
		if [query? hasprop refkey] {
		    output [query propval refkey]
		} else {
		    output "???"
		}
	    }
	    output {]}
	}
    }

    {el} {
	content {
	    if [$spec has result] { startTag [$spec get result] }
	    output [$spec get prefix ""]
	    doChildren
	    output [$spec get suffix ""];
	    if [$spec has result] { endTag [$spec get result] } else checkUndef
	}
    }
    {} 	{ content { doChildren } }
}

proc checkUndef {} {
    global _undefGIs spec
    if [info exists _undefGIs([q gi])] return
    if {[$spec has prefix] || [$spec has suffix]} {
	set _undefGIs([q gi]) 1
    } else {
	puts stderr "Warning: no rule for [q gi]"
	set _undefGIs([q gi]) 0
    }
}



proc doChildren {} {
    global spec
    foreachNode child { $spec do content }
}
proc doNode {} {
    global spec
    $spec do content
}

global spec
set spec tohtml

###

proc convert {} {
    global curnode spec
    withNode docroot subtree element HEAD {
	withNode subtree element TITLE {
	    set title [content]
	}
    }
    set curnode [HTMLNode #auto -title $title]
    withNode doctree element HEAD child element AUTHOR hasatt EMAIL {
	$curnode addLink REV MADE HREF "mailto:[q attval EMAIL]"
    }
    $curnode start
    $spec do content
    $curnode finish
}

proc main {} {
    parseargs
    setprops
    convert
}

proc parseargs {} {
    global argv
    while {[llength $argv]} {
	switch -- [set opt [shift argv]] {
	    "-ednotes"  { global includeEdnotes; set includeEdnotes 1 }
	    "-graphics"  { global includeGraphics; set includeGraphics 1 }
	    default {
		puts stderr "Unrecognized option $opt"
	    }
	}
    }
}

proc minitoc {depth} {
    set baselevel [query ancestor propval seclevel]
    if { $baselevel == "" } { set baselevel 0 }
    set stoplevel [expr $baselevel + $depth]
    if {$depth <= 0} {
	incr stoplevel 99
    }
    set curlevel $baselevel

    foreachNode descendant element SECT {
	set newlevel [query propval seclevel]
	if {$newlevel <= $stoplevel} {
	    while {$curlevel < $newlevel} {
		nl; startTag UL
		incr curlevel
	    }
	    while {$curlevel > $newlevel} {
		nl; endTag UL
		incr curlevel -1
	    }
	    nl
	    startTag LI
	    startTag A HREF "#[query propval reflabel]"
	    withNode child element TITLE {
		cdata "[query parent propval refnumber] [content]"
	    }
	    endTag A
	}
    }
    while {$curlevel > $baselevel} {
	nl;
	endTag UL
	incr curlevel -1
    }
}
