# File: specs/latex.spec
# costdoc->LaTeX specification
#

require specs/Latex.tcl

global includeEdnotes; set includeEdnotes 0

proc parseargs {} {
    global argv
    while {[llength $argv]} {
	switch -- [set opt [shift argv]] {
	    "-ednotes"  { global includeEdnotes; set includeEdnotes 1 }
	    default {
		puts stderr "Unrecognized option $opt"
	    }
	}
    }
}

proc main {} { 
    parseargs
    process tolatex 

}


LatexProcess tolatex -specification {
    {element COSTDOC} {
	arcform 	document
	documentclass	article
	classoptions	{}
	packages	{t1enc alltt times myfmt}
	preamble	{ [makePreamble] }
	prefix 		{\maketitle\tableofcontents}
    }

    {element P} 	{ arcform paragraph }
    {element SECT} 	{ arcform division }
    {element TITLE in SECT} { arcform divheading }
    {element H} 	{ arcform macro   macro "paragraph*" }

    {elements "EMPH DFN DPH"} {
	arcform macro
	macro "emph"
    }

    {elements "CMD SUBCMD QC ENV NODETYPE EVENT CLASS ATTR ELEM"} {
	arcform macro
	macro "texttt"
    }

    {element TAG} {
	arcform macro
	macro texttt
	prefix {<}
	suffix {>}
    }

    {element ENVAR} {
	arcform macro
	macro texttt
	prefix {\$}
    }

    {elements "OPTION OPTARG ARG"} { arcform macro }
    {element OPTION} 	{ macro textbf }
    {element OPTARG} 	{ macro textit }
    {element ARG} 	{ macro textit }

    {element LIT} {
	arcform macro
	macro "texttt"
    }
    {elements "SAMP"} {
	arcform macro
	macro "texttt"
	before "`"
	after "'"
    }

    {element CMDDEF} 		{ arcform environment  envname "framed" }
    {elements "TIP NOTE"}	{ arcform environment  envname "quote" }

    {element DLIST} 		{ arcform list   envname "description" }
    {element DT}		{ arcform itemtag }
    {element DD} 		{ arcform itembody }

    {element LIST} 		{ arcform list  envname "itemize" }
    {element ENUM} 		{ arcform list  envname "enumerate" }
    {element IT}		{ arcform item }

    {elements "SYNOPSIS SYNTAX"} { 
	arcform verbatim
	envname alltt
	cdataFilter allttFilter
    }
    {elements "EXAMPLE LISTING" hasatt SOURCE} {
	arcform special
	before {\begin{verbatim}}
	content {[readFile [query entity [query attval SOURCE] sysid]]}
	suffix {\end}
	after  {{verbatim}}
    }

    {elements "EXAMPLE LISTING"} {
	arcform verbatim 
	envname alltt
	cdataFilter allttFilter
    }

    {element EDNOTE} {
	arcform	special
	envname ednote
	startAction {
	    global includeEdnotes
	    if $includeEdnotes environmentStart else ignoreStart
	}
	endAction {
	    global includeEdnotes
	    if $includeEdnotes environmentEnd else ignoreEnd
	}
    }

    {elements "HEAD TITLEPG"} { arcform ignore }
}

#
# Extra utilities:
#

proc makePreamble {} {
    set txt ""
    withNode docroot subtree element HEAD {
	withNode child element title  { append txt "\\title{[content]}\n" }
	withNode child element author { append txt "\\author{[content]}\n" }
	withNode child element date   { append txt "\\date{[content]}\n" }
    }
    return $txt
}

proc readFile {filename} {
    # %%% for BSD systems, use '|expand' instead of '|newform ...'
    if [catch {set fp [open "|newform -i-8 -o-0 $filename" r]}] {
	warning "Can't process $filename"
	return;
    }
    set content [read $fp]
    close $fp
    return $content
}

# %%% doesn't do tabs quite right
substitution allttFilter {
    "\{"	"\\\{"
    "\}"	"\\\}"
    "\\"	"\\char\"5C{}"
    "\t"	"        "
}

