#
# outline.spec
# Build a table of contents and command/query clause index
# from the main document.
#

proc main {args} {
#
# The first pass prints the table of contents 
# and annotates each SECT element with properties 
# that are used in the second pass:
#
    process outline
    nl; nl;
#
# The second pass builds and prints an index of each command
# (CMD elements) and query clause (QC elements) used in the document,
# printing the section number(s) where they appear. 
#
    puts stdout "Commands:"
    listall CMD
    nl;
    puts stdout "Query clauses:"
    listall QC
}


#
# Pass 1: table of contents
# Lists all <SECT>ion <TITLE>s and <H>eadings,
# assigning section number properties ('secnum') on the way.
#
global secdepth ;	# current nesting level
global secctrs ;	# array: secdepth -> current section number

set secdepth 1
set secctrs($secdepth) 0

specification outline {

    {element SECT} {
	START {
	    global secdepth secctrs
	    incr secctrs($secdepth)

	    # Set node properties:
	    setprop secdepth $secdepth
	    setprop secctr $secctrs($secdepth)
	    setprop secnum [join [query* rootpath propval secctr] "." ]

	    # Set up for subsections:
	    incr secdepth
	    set secctrs($secdepth) 0
	}
	END {
	    incr secdepth -1
	}
    }

    {element TITLE} {
	START {
	    global secdepth
	    indent $secdepth
	    output "[query parent propval secnum] "
	}
	END {
	    nl
	}
    }
    {textnode within TITLE} {
	CDATA { output [content] }
    }

    {element H} {
	START { indent [expr $secdepth + 1] }
	END { nl }
    }
    {textnode within H} {
	CDATA { output [content] }
    }
}


#
# Pass 2: build and print an index of terms.
# 'gi' is the generic identifier of the element to be indexed.
#
proc listall {gi} {
    foreachNode doctree element $gi {
	set term [content]
	set where [query ancestor propval secnum]
	lappend tindex($term) $where
    }

    foreach term [lsort [array names tindex]] {
	set tindex($term) [luniq $tindex($term)]
	indent 1
	output "$term ([join $tindex($term) ", "])";  nl
    }
}

#
# Miscellaneous utilities:
#
proc output {data} { puts stdout $data nonewline; }
proc nl {} { puts stdout ""; }
proc indent {n} {
    while {$n > 0} {
	output "    ";
	incr n -1;
    }
}
