/***********************************************************************
 *               Copyright (C) 1995 Joe English
 *                   Freely redistributable
 ***********************************************************************
 *
 * File:	esisp.h
 * Author: 	Joe English
 * Created: 	Jan 1995
 * Description: Private definitions for libesis.
 *
 * Requires:	stdio.h, strmap.h, pile.h
 *
 * $Date: 95/11/29 15:11:58 $
 * $Revision: 1.10 $
 * $Locker:  $
 */

#ifndef ESISP_H
#define ESIS_H 1

struct ESISNode
{
    ESISNodeType	type;		/* type of this node */
    ESISToken		name;		/* GI, or attribute or entity name */
    ESISNode		attributes;	/* attributes & auxilliary nodes */
    strmap		properties;	/* application properties */

    /* Content: */
    ESISNode		children;	/* container nodes: first child */
    ESISNode		reference;	/* reference nodes: referenced node */
    ESISData		text; 		/* data nodes: character data */

    /* Tree structure: */
    ESISNode		parent;		/* parent or owner node */
    ESISNode		next;		/* next child/attribute */
    ESISNode		prev;		/* previous child/attribute */
    ESISNode		link;		/* ilink structure; see relation.c */

    /* Pathloc document position: */
    long		pathno, width;
    short		height, depth;	
};

/* Entities:
 * Replacement text of internal entities stored in child nodes 
 * %%%need: internal or external flag, entity type
 * Attributes:
 * if 'text' is NULL, attribute has #IMPLIED value. 
 * %%%need(?): attribute declared value/attribute type.
 * %%%need: LPD name for link attributes. 
 */

/*
 * Internal ESIS property names:
 * All begin with an RNI '#', to keep user's namespace free.
 */

/* External entities, subdocuments: */
#define ENTPROP_SYSID		"#sysid"	/* system identifier */
#define ENTPROP_PUBID		"#pubid"	/* public identifier */
/* External data entities: */
#define ENTPROP_NOTATION	"#notation"	/* notation for entities */
#define ENTPROP_NOTATION_SYSID	"#notsysid"	/* ntn system identifier */
#define ENTPROP_NOTATION_PUBID	"#notpubid"	/* ntn public identifier */

/* ESIS streams:
 */

typedef struct ESISRelation *ESISRelation;	/* See relation.c */

struct ESISStream
{
    ESISNode	rootnode;	/* document root */
    ESISRelation relations;	/* application-defined relations */

    /* Output stream (returned to caller) pointers: */
    ESISNode	rdptr;		/* next node to be delivered to caller */

    /* Input stream (reading from file) pointers: */
    ESISNode 	curnode;	/* ??? %%% wrcurnode */
    ESISNode	wrptr;		/* ... */

    pile	datapile;	/* for data storage */
    FILE * 	sgmlsinput;	/* SGMLS input stream */

    /* Pathloc counter: */
    long	pathno;		/* current path number */


    /* need: 
     * doc x name->entity map,
     * doc x name->notation map,
     * current subdocument
     * Others???
     */
};

/* Node construction functions: */
extern ESISNode esis_open_node(ESISStream, ESISNodeType);
extern ESISNode esis_close_node(ESISStream);
extern ESISNode esis_create_datanode
	(ESISStream ep, ESISNodeType type, char *text);
extern ESISNode esis_create_attribute(ESISNode, ESISToken name, char *attval);
extern ESISNode esis_create_entity(ESISStream, ESISToken name);
extern ESISNode esis_find_entity(ESISStream, ESISToken name);

/* low-level: */
extern ESISNode esis_create_node(ESISNodeType, ESISNode parent, ESISNode esib);


#endif /* ESISP_H */

