/***********************************************************************
 *               Copyright (C) 1995 Joe English
 *                   Freely redistributable
 ***********************************************************************
 *
 * File:	project.h
 * Author:	Joe English
 * Created:	Feb 1994
 * Description:	Useful macros used in Cost implementation.
 *
 * $Date: 96/01/16 12:42:22 $
 * $Revision: 1.3 $
 * $Locker:  $
 */


#ifndef _PROJECT_H
#define _PROJECT_H 1

/*
 * Assertions: Run-time checking of invariant conditions.
 * Usage: ASSERT(test, message)
 * Assertions are only turned on if -DDEBUG is used.
 * Failed assertions print the message and abort().
 * This is different from the standard assert() in that
 * it is turned *off* by default, and the programmer
 * must supply a documentary message explaining the assertion.
 */

/* !!! Fix this... */
#if DEBUG
#	ifndef BUFSIZ
#		include <stdio.h>
#	endif
#    define ASSERT(test,message) 				\
	if (!(test)) {						\
	    fprintf(stderr,"Assertion failed: %s line %d\n%s",		\
		__FILE__, __LINE__, message);			\
	    abort();						\
	}
#else
#    define ASSERT(test,message)
#endif

#if DBMALLOC
#	include "dbmalloc.h"
#endif

/* STOMP value is used for debugging purposes;
 * freed pointers may be set to STOMP (0x1) 
 * so any future use will crash the program sooner rather than later.
 */
#ifndef STOMP
#	define STOMP ((void *)0x1)
#endif
#ifndef STOMPON
#    if DEBUG
#	define STOMPON(p) (p)=((void *)0x1);
#    else
#	define STOMPON(p)
#    endif
#endif

#endif		/* _PROJECT_H	*/
