/* 
 * tclAppInit.c and main() routine for Cost
 * This file is used for both costsh (if -DUSE_TK is not defined)
 * and for costwish (if it is.)
 *
 * Got a favorite Tcl extension that I've left out?  Let me know...
 *
 * NB: Not all combinations have been tested.
 *
 * Joe English <joe@art.com>
 */

#ifndef USE_ITCL	/* recommend: version 2.0 */
#define USE_ITCL 1
#endif
#ifndef USE_TCLX
#define USE_TCLX 0
#endif
#ifndef USE_TK		/* require: version > 4.0 */
#define USE_TK 0
#endif
#ifndef USE_BLT		/* require: version > 1.8 */
#define USE_BLT 0
#endif

#include "tcl.h"

#if USE_TCLX
#	include "tclExtend.h"
#endif
#if USE_ITCL
#	include "itcl.h"
#endif
#if USE_TK
#	include "tk.h"
#endif

/* Declarations for various optional extensions w/no headers:  */

extern int Blt_Init(Tcl_Interp *);	/* #if USE_BLT */
extern int Tix_Init(Tcl_Interp *);	/* #if USE_TIX */

extern int Cost_Init(Tcl_Interp *);

#define HAVEEXTENSION(interp, name) \
	Tcl_SetVar2(interp, "COST", "EXTENSIONS", name, \
		TCL_GLOBAL_ONLY | TCL_LIST_ELEMENT | TCL_APPEND_VALUE);


int Tcl_AppInit(Tcl_Interp *interp)
{
    if (Tcl_Init(interp) != TCL_OK)
	return TCL_ERROR;

#if USE_TCLX
    if (TclX_Init(interp) != TCL_OK) return TCL_ERROR;
    HAVEEXTENSION(interp, "TCLX")
#endif

#if USE_ITCL
    if (Itcl_Init(interp) != TCL_OK) return TCL_ERROR;
    HAVEEXTENSION(interp, "ITCL")
#endif 

#if USE_TK
    if (Tk_Init(interp) == TCL_ERROR) return TCL_ERROR;
    HAVEEXTENSION(interp, "TK");

/*
 * Tk-only extensions:
 */

#if USE_BLT
    if (Blt_Init(interp) != TCL_OK) return TCL_ERROR;
    HAVEEXTENSION(interp, "BLT");
#endif
#if USE_TIX
    if (Tix_Init(interp) != TCL_OK) return TCL_ERROR;
    HAVEEXTENSION(interp, "TIX");
#endif

#endif	/* USE_TK */

    if (Cost_Init(interp) != TCL_OK) return TCL_ERROR;
    HAVEEXTENSION(interp, "COST");

#if !USE_TK
    if (Tcl_Eval(interp, "cost_commandline") != TCL_OK) return TCL_ERROR;
#endif

    tcl_RcFileName = 0;
    return TCL_OK;
}

int main(int argc, char *argv[])
{
#if USE_TK
    Tk_Main(argc, argv, Tcl_AppInit);
#else
    Tcl_Main(argc, argv, Tcl_AppInit);
#endif
    return 0;
}
