/***********************************************************************
 *               Copyright (C) 1995 Joe English
 *                   Freely redistributable
 ***********************************************************************
 *
 * File:	tclcost.c
 * Author:	Joe English
 * Created: 	28 Feb 1995
 * Description:	CoST/Tcl initialization and interface routines
 */

#ifndef lint
static char RCSId[] = "$Id: tclcost.c,v 1.33 96/01/16 12:46:54 joe Exp $";
#endif

#define COST_VERSION_MAJOR "2"
#define COST_VERSION_MINOR "0"
#define COST_VERSION "2.0a3"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tcl.h"

#include "project.h"
#include "strmgt.h"
#include "esis.h"
#include "costq.h"
#define COSTGLOBAL
#include "tclcost.h"

/*
 * CostEval: Evaluate Tcl expression with current node
 */

int CostEval(Tcl_Interp *interp, ESISNode nd, char *expr)
{
    ESISNode old_node = current_node;
    int status;

    current_node = nd;
    status = Tcl_Eval(interp, expr);
    current_node = old_node;
    return status;
}

/*+++
 * Command procedures:
 */
static CMDPROC(CostLoadProc)
{
    char *filehandle;
    FILE *fp;
    int status;
    ESISEventType eventType;
    ESISNode evnode;

    CHECKNARGS(1, "filehandle")

    filehandle = argv[1];

    status = Tcl_GetOpenFile(interp, filehandle,
	0,	/* forWriting */
	1, 	/* checkUsage */
	&fp);

    if (status != TCL_OK)
	return TCL_ERROR;

    if (current_document)
	estream_close(current_document);
    current_node = 0;

    /* Load the whole document: */
    current_document = estream_create(fp);
    do {
	eventType = estream_next(current_document, &evnode);
    } while (eventType != EV_EOF && eventType != EV_ERROR);

    if (eventType == EV_EOF) {
	current_node = estream_rootnode(current_document);
	return TCL_OK;
    } else {
	Tcl_AppendResult(interp, "Error reading ", filehandle, NULL);
	return TCL_ERROR;
    }
}

static CQStatus CostQueryContinuation(
	ESISNode nd, const char *value, void *closure)
{
    Tcl_DString *dstr = (Tcl_DString *)closure;
    if (value) {
	Tcl_DStringAppend(dstr, (/*!const*/char *)value, -1);
    }
    return CQ_SUCCEED;
}

/*ARGSUSED*/
static CQStatus CostQueryAllContinuation(
	ESISNode nd,const char *value,void *closure)
{
    Tcl_DString *dstr = (Tcl_DString *)closure;
    if (value)
	Tcl_DStringAppendElement(dstr, (/*!const*/char *)value);
    return CQ_FAIL;	/* force query continuation */
}

/*ARGSUSED*/
static CQStatus CostQueryCountContinuation(
	ESISNode nd,const char *value,void *closure)
{
    int (*ip) = (int *)closure;
    ++(*ip);
    return CQ_FAIL;	/* force query continuation */
}

static CMDPROC(CostQueryProc)	/* %%% factor */
{
    CQQuery q;
    Tcl_DString ans;

    CHECKCURNODE
    /* %%% #args? */

    Tcl_DStringInit(&ans);
    q = cq_buildquery(argv+1, argc - 1);
    if (!q)
	return TCL_ERROR;
    (void)cq_doquery(current_node, q, CostQueryContinuation, &ans);
    cq_destroyquery(q);
    Tcl_ResetResult(interp);	/* bug in Tcl */
    Tcl_DStringResult(interp, &ans);
    return TCL_OK;
}

static CMDPROC(CostQueryAllProc)	/* %%% factor */
{
    CQQuery q;
    Tcl_DString ans;
    /* %%% #args? */
    CHECKCURNODE
    Tcl_DStringInit(&ans);
    q = cq_buildquery(argv+1, argc - 1);
    if (!q)
	return TCL_ERROR;
    (void)cq_doquery(current_node, q,
	CostQueryAllContinuation, &ans);
    cq_destroyquery(q);
    Tcl_ResetResult(interp);	/* bug in Tcl */
    Tcl_DStringResult(interp, &ans);
    return TCL_OK;
}

static CMDPROC(CostMatchProc)	/* %%% factor w/CostQueryProc */
{
    CQQuery q;
    int status;
    /* %%% #args? */
    CHECKCURNODE
    q = cq_buildquery(argv+1, argc - 1);
    if (!q)
	return TCL_ERROR;
    status = cq_testquery(current_node, q);
    cq_destroyquery(q);
    Tcl_SetResult(interp, status ? "1" : "0", TCL_STATIC);
    return TCL_OK;
}

static CMDPROC(CostCountProc)	/* %%% factor */
{
    CQQuery q;
    int count;
    char buf[16];
    CHECKCURNODE
    q = cq_buildquery(argv+1, argc - 1);
    if (!q) return TCL_ERROR;
    count = 0;
    (void)cq_doquery(current_node, q, CostQueryCountContinuation, &count);
    cq_destroyquery(q);
    sprintf(buf, "%d", count);
    Tcl_SetResult(interp, buf, TCL_VOLATILE);
    return TCL_OK;
}


  /*ARGSUSED*/
static CQStatus CostSelectNodeContinuation(
	ESISNode node, const char *value,  void *closure)
{
    if (node && !value)
	current_node = node;
    return CQ_SUCCEED;
}

static CMDPROC(CostSelectNodeProc)	/* %%% factor w/CostQueryProc */
{
    CQQuery q;
    CQStatus status;

    /* %%% #args? */
    CHECKCURNODE
    q = cq_buildquery(argv+1, argc - 1);
    if (!q)
	return TCL_ERROR;
    status = cq_doquery(current_node, q, CostSelectNodeContinuation, 0);
    cq_destroyquery(q);
    Tcl_SetResult(interp, status == CQ_SUCCEED ? "1" : "0", TCL_STATIC);
    return TCL_OK;
}

/* implementation of withNode and foreachNode 
*/

typedef struct 
{
    Tcl_Interp	*interp;
    char	*script;
    CQStatus	cq_default_status;
    int 	status;		/* Tcl result status */
} CQData;


  /*ARGSUSED*/
static CQStatus CostNodeLoopContinuation(
	ESISNode node, const char *value, void *closure)
{
    CQData *cqdata = (CQData *)closure;
    if (node && !value) 
    {
	int status = CostEval(cqdata->interp, node, cqdata->script);
	switch (status) {
	    case TCL_CONTINUE:
	    case TCL_OK:
		break;
	    case TCL_BREAK:
		status = TCL_OK;
		/*FALLTHRU*/
	    case TCL_ERROR :
	    default:
		cqdata->status = status;
		return CQ_SUCCEED;	/* stop evaluating query %%% */
	}
    }
    return cqdata->cq_default_status;
}

static CMDPROC(CostNodeLoopProc)
{
    CQQuery q;
    CQData cqdata;
    int which = (int)clientData;
    char *script = argv[argc - 1];

    if (argc < 3) {
	Tcl_AppendResult(interp, "wrong #args\n", NULL);
	Tcl_AppendResult(interp, "Usage: ", argv[0], " query... cmd", NULL);
	return TCL_ERROR;
    }
    CHECKCURNODE

    q = cq_buildquery(argv+1, argc - 2);
    if (!q) return TCL_ERROR;

    cqdata.interp = interp;
    cqdata.script = script;
    cqdata.cq_default_status =  which ? CQ_FAIL : CQ_SUCCEED;
	/* 1: keep trying; 0: only once %%% */
    cqdata.status = TCL_OK;

    (void)cq_doquery(current_node, q, CostNodeLoopContinuation , &cqdata);
    cq_destroyquery(q);
    return cqdata.status;
}

static CMDPROC(CostContentProc)
{
    ESISString content;

    CHECKCURNODE
    CHECKNARGS(0,"");

    content = esis_text(current_node);
    if (content)
    {
	Tcl_SetResult(interp, content, TCL_VOLATILE);
    } 
    else
    {	/* Not a data node -- concatenate character data of descendants */ 
	ESISNode nd = esis_firstpreorder(current_node);
	Tcl_ResetResult(interp);
	while (nd) 
	{
	    ESISNodeType ndtype = esis_nodetype(nd);
	    if (ndtype == EN_CDATA || ndtype == EN_RE)
		Tcl_AppendResult(interp, esis_text(nd), NULL);
	    nd = esis_nextpreorder(current_node, nd);
	}
    }
    return TCL_OK;
}

static CMDPROC(CostSetpropProc)
{
    char *propname, *propval;
    CHECKNARGS(2,"propname propval");
    CHECKCURNODE
    propname = argv[1];
    propval = argv[2];
    esis_setprop(current_node, propname, propval);
    return TCL_OK;
}

static CMDPROC(CostUnsetpropProc)
{
    int i;
    if (argc < 1) {
	Tcl_AppendResult(interp,
		"Usage: ",argv[0]," propname [propname..]",NULL);
	return TCL_ERROR;
    }
    CHECKCURNODE
    for (i=1; i<argc; ++i)
	esis_unsetprop(current_node, argv[i]);
    return TCL_OK;
}

/* +++ Ilinks and relations:
 */

static CMDPROC(CostRelationProc)
{
    ESISToken relname;
    char **anchnames;
    int nanchors;

    CHECKCURNODE

    if (argc <= 1) {
	Tcl_AppendResult(interp, 
		"Usage: ", argv[0], " relname [anchname ...]",
		NULL);
	return TCL_ERROR;
    }

    relname = ucintern(argv[1]);
    anchnames = argv + 2;
    nanchors = argc - 2;
    if (!esis_define_relation(current_document, relname, nanchors, anchnames)) {
	Tcl_AppendResult(interp, "Error defining relation ", relname, NULL);
	return TCL_ERROR;
    }
    return TCL_OK;
}

static CMDPROC(CostAddLinkProc)
{
    ESISToken relname;
    ESISNode ilink;
    ESISNode old_current_node = current_node;

    if (argc <  1 || argc % 2 != 0) {
	Tcl_AppendResult(interp, 
	    "Usage: ", argv[0], " relname [ anchname \"query...\" ... ]", 
	    NULL);
	return TCL_ERROR;
    }

    ++argv; --argc;
    relname = ucintern(argv[0]);
    ++argv; --argc;

    ilink = esis_create_ilink(current_document, relname, current_node);
    if (!ilink) {
	Tcl_AppendResult(interp, "Error adding ", relname, " link",NULL);
	return TCL_ERROR;
    }

    while (argc > 0) {
	ESISToken anchname;
	char **qargs;
	int nqargs;
	CQQuery q;
	CQStatus status;
	ESISNode anchor;

	anchname = ucintern(argv[0]); 
	if (TCL_OK != Tcl_SplitList(interp, argv[1], &nqargs, &qargs))
	    return TCL_ERROR;

	q = cq_buildquery(qargs, nqargs);
	free(qargs);
	if (!q) return TCL_ERROR;
	status = cq_doquery(ilink, q, CostSelectNodeContinuation, 0);
	anchor = current_node; current_node = old_current_node;
	cq_destroyquery(q);
	if (status == CQ_SUCCEED) {
	    if (!esis_set_linkend(ilink, anchname, anchor)) {
		Tcl_AppendResult(interp, 
		    "Error creating ", anchname," anchor in ", relname,
		    NULL);
		return TCL_ERROR;
	    }
	}
	argv += 2; argc -= 2;
    }
    return TCL_OK;

}

/* +++ Handlers
 * CommandEventHandler: closure is a Tcl cmdInfo structure.
 * Invokes Tcl proc with one argument, the event type name.
 */

typedef struct 
{
    Tcl_Interp	*interp;
    Tcl_CmdInfo	cmdInfo;
    const char	*cmdName;
    int 	status;		/* return status from handler */
} EHData;

static int CommandEventHandler
	(ESISEventType ev, ESISNode node, void *closure)
{
    ESISNode old_node = current_node;
    EHData *ehdata = (EHData *)closure;
    const char *evname = esis_evtype_name(ev);
    const char *argv[3];
    int argc = 2;
    int status;

    argv[0] = ehdata->cmdName;
    argv[1] = evname;
    argv[2] = 0;
    current_node = node;
    Tcl_ResetResult(ehdata->interp);	/* another Tcl 7.3 bug %%% */
    status = (ehdata->cmdInfo.proc)
	(ehdata->cmdInfo.clientData, ehdata->interp, argc,
	(/*!const*/char **)argv);
    current_node = old_node;
    if (status == TCL_ERROR) {
	Tcl_AppendResult(ehdata->interp, "\nError in ",evname," handler",NULL);
	if (esis_gi(node))
	    Tcl_AppendResult(ehdata->interp,
		" (", esis_gi(node), " element)", NULL);
    }
    ehdata->status = status;
    return status == TCL_OK;
}

static CMDPROC(CostProcessProc)
{
    char *cmdName;
    EHData ehdata;

    CHECKCURNODE
    CHECKNARGS(1, "handler")
    cmdName = argv[1];
    if (!Tcl_GetCommandInfo(interp, cmdName, &ehdata.cmdInfo)) {
	Tcl_AppendResult(interp,argv[0],": no proc ",argv[1],NULL);
	return TCL_ERROR;
    }
    ehdata.interp = interp;
    ehdata.cmdName = cmdName;
    ehdata.status = TCL_OK;

    esis_traverse(current_node, CommandEventHandler, &ehdata);
    return ehdata.status;
}

static CMDPROC(CostDefineSpecificationProc)
{
    CQAssoc assoc;

    CHECKNARGS(2,"name alist");
    assoc = assocConstructor(interp, argv[2]);
    if (!assoc) {
	Tcl_AppendResult(interp, "Error defining ", argv[1], NULL);
	return TCL_ERROR;
    }

    Tcl_CreateCommand(interp,argv[1],assocProc, 
	(ClientData)assoc, assocDestructor);

    Tcl_SetResult(interp, argv[1], TCL_VOLATILE);
    return TCL_OK;
}

/*+++ Event handler routines:
*/

typedef struct 
{
    Tcl_Interp	*interp;		
    int		status;			/* return status */
    char 	**handlerScripts;	/* array [ESISEventTypeMax]; */
} CostEventHandlerRec;

static int CostEventHandler(ESISEventType ev, ESISNode node, void *closure)
{
    CostEventHandlerRec *tr = (CostEventHandlerRec *)closure;
    char *expr = tr->handlerScripts[ev];

    if (expr!= NULL) 
    {
	ESISNode old_current_node = current_node;
	int status;

	current_node = node;
	if (tr->handlerScripts[ESISEventTypeMax] == NULL) /* %%% Blech */
	    status = Tcl_Eval(tr->interp, expr);
	else 
	    status = Tcl_GlobalEval(tr->interp, expr);
	current_node = old_current_node;

	switch (status) 
	{
	    case TCL_OK: 	
		return 1;
	    case TCL_BREAK:	
		status = TCL_OK;
		/* FALLTHRU */
	    default:
		tr->status = status;
		return 0;
	}
    } 
    /* else */
    return 1;
}

static CMDPROC(CostEventHandlerProc)
{
    CostEventHandlerRec tr;

    CHECKCURNODE

    tr.interp = interp;
    tr.handlerScripts = (char **)clientData;
    tr.status = TCL_OK;

    if (argc == 1) 
    {
	esis_traverse(current_node, CostEventHandler, &tr);
    }
    else if (argc == 2)	/* called as an event handler from 'process' */
    {
	ESISEventType ev = esis_string_to_evtype(argv[1]);
	if (ev == EV_ERROR) {
	    Tcl_AppendResult(interp, "\nBad event name ", argv[1], NULL);
	    return TCL_ERROR;
	}
	CostEventHandler(ev, current_node, &tr);
    }
    else 
    {
	Tcl_AppendResult(interp, "Usage: ", argv[0], NULL);
	return TCL_ERROR;
    }

    return tr.status;

}

static void CostEventHandlerDestructor(void *closure) 
{
    char **handlerScripts = (char **)closure;
    int i;
    for (i=0; i<ESISEventTypeMax; ++i)
	if (handlerScripts[i])
	    free(handlerScripts[i]);
    free(handlerScripts);
}

static CMDPROC(CostDefineEventHandler)
{
    char **handlerScripts, **speclist;
    char *procname, *handlers;
    int i, speclistlen, status;
    int isglobal = 0;

    if (argc < 3) {
usage:
	Tcl_AppendResult(interp, "Usage: ", argv[0],
	    " procname [ -global ] { EVENT script ... }", NULL);
	return TCL_ERROR;
    }

    i = 1;
    procname = argv[i]; ++i;

    if (*argv[i] == '-') 
    {
	if (!strcmp(argv[i], "-global"))	
		isglobal = 1;
	else
		goto usage;
	++i;
    }
    if (i != argc - 1)
	goto usage;
    handlers = argv[i];

    if (TCL_OK != Tcl_SplitList(interp, handlers, &speclistlen, &speclist))
	return TCL_ERROR;

    handlerScripts = malloc((ESISEventTypeMax + 1) * sizeof(char *));
    for (i=0; i<ESISEventTypeMax; ++i) 
	handlerScripts[i] = NULL;
    handlerScripts[ESISEventTypeMax] = isglobal ? "global" : 0;	/* %%% */

    Tcl_SetResult(interp, procname, TCL_VOLATILE);
    status = TCL_OK;
    if (speclistlen % 2) {
	--speclistlen;
	status = TCL_ERROR;
    }
    for (i=0; i<speclistlen; i += 2) {
	ESISEventType ev = esis_string_to_evtype(speclist[i]);
	if (ev == EV_ERROR) {
	    Tcl_AppendResult(interp, "\nBad event name ", speclist[i], NULL);
	    status = TCL_ERROR;
	}
	else if (handlerScripts[ev]) {
	    Tcl_AppendResult(interp, "\n event multiply specified",
		speclist[i], NULL);
	    status = TCL_ERROR;
	}
	else {
	    handlerScripts[ev] = savestring(speclist[i+1]);
	}
    }

    Tcl_CreateCommand(interp,procname, CostEventHandlerProc, 
	(ClientData)handlerScripts, CostEventHandlerDestructor);

    free(speclist);
    return status;
}

/*+++
 *----------------------------------------------------------------------
 *
 * Cost_Init --
 *	Package initialization for CoST
 *	Register commands, load initialization file.
 *
 *----------------------------------------------------------------------
 */

int Cost_Init(Tcl_Interp *interp)
{
    int status;
    Tcl_DString startup_file;

    Tcl_SetVar2(interp, "COST", "VERSION", COST_VERSION, TCL_GLOBAL_ONLY);
    Tcl_SetVar2(interp, "COST", "MAJOR", COST_VERSION_MAJOR, TCL_GLOBAL_ONLY);
    Tcl_SetVar2(interp, "COST", "MINOR", COST_VERSION_MINOR, TCL_GLOBAL_ONLY);

    /* Register Cost commands: */
    Tcl_CreateCommand(interp, "loadsgmls", 	CostLoadProc, NULL, NULL);
    Tcl_CreateCommand(interp, "query",   	CostQueryProc, NULL, NULL);
	Tcl_CreateCommand(interp, "q",   	CostQueryProc, NULL, NULL);
    Tcl_CreateCommand(interp, "queryall", 	CostQueryAllProc, NULL, NULL);
	Tcl_CreateCommand(interp, "query*", 	CostQueryAllProc, NULL, NULL);
	Tcl_CreateCommand(interp, "q*", 	CostQueryAllProc, NULL, NULL);
    Tcl_CreateCommand(interp, "query?",  	CostMatchProc, NULL, NULL);
	Tcl_CreateCommand(interp, "q?",  	CostMatchProc, NULL, NULL);
    Tcl_CreateCommand(interp, "countq",  	CostCountProc, NULL, NULL);
    Tcl_CreateCommand(interp, "withNode",
				CostNodeLoopProc,(ClientData)0, NULL);
    Tcl_CreateCommand(interp, "foreachNode",
				CostNodeLoopProc,(ClientData)1, NULL);

    Tcl_CreateCommand(interp, "content",  	CostContentProc, NULL, NULL);
    Tcl_CreateCommand(interp, "setprop",  	CostSetpropProc, NULL, NULL);
    Tcl_CreateCommand(interp, "unsetprop",  	CostUnsetpropProc, NULL, NULL);

    Tcl_CreateCommand(interp, "specification", 	CostDefineSpecificationProc,
								NULL,NULL);
    Tcl_CreateCommand(interp, "process",   	CostProcessProc, NULL, NULL);
    Tcl_CreateCommand(interp, "eventHandler", 	CostDefineEventHandler,
								NULL, NULL);
    Tcl_CreateCommand(interp, "selectNode", 	CostSelectNodeProc, NULL, NULL);

    Tcl_CreateCommand(interp, "relation", 	CostRelationProc, NULL, NULL);
    Tcl_CreateCommand(interp, "addlink", 	CostAddLinkProc, NULL, NULL);

    /* Misc: */
    Tcl_CreateCommand(interp, "substitution", 	DefineSubstProc,NULL,NULL);
    Tcl_CreateCommand(interp, "environment",
		DefineEnvironmentProc,NULL,NULL);

    /*
     * Load startup file:
     */
    Tcl_DStringInit(&startup_file);
    if (getenv("COSTLIB")) {
	Tcl_DStringAppend(&startup_file, getenv("COSTLIB"), -1);
	Tcl_DStringAppend(&startup_file, "/", 1);
    }
    Tcl_DStringAppend(&startup_file, "costinit.tcl", -1);
    status = Tcl_EvalFile(interp, Tcl_DStringValue(&startup_file));
    if (status != TCL_OK) {
	char cmd[] = "puts stderr $errorInfo\n";
	fprintf(stderr,"Error: %s\n", interp->result);
	(void)Tcl_Eval(interp, cmd);
	exit(1);
    }

    return status;
}

