#
#  fmt_html.pl
#
#  $Id: fmt_html.pl,v 1.7 1997/05/21 10:06:29 cg Exp $
#
#  HTML-specific driver stuff
#
#   Copyright 1996, Cees de Groot
#

require "$LibDir/fixref.pl";
$fixref = $fixref::fixref;

require "$LibDir/html2html.pl";
$html2html = $html2html::html2html;

$html = {};
$html->{NAME} = "html";
$html->{HELP} = "";
$html->{OPTIONS} = [
   { option => "split", type => "l", 
     'values' => [ "0", "1", "2" ], short => "s" },
   { option => "dosnames", type => "f", short => "D" },
   { option => "imagebuttons", type => "f", short => "i"}
];
$html->{'split'}  = 1;
$html->{dosnames}  = 0;
$html->{imagebuttons}  = 0;
$html->{preNSGMLS} = << 'EOF';
  $NsgmlsOpts .= " -ifmthtml ";
EOF
$html->{preASP} = html_preASP;
$html->{postASP} = html_postASP;

$Formats{$html->{NAME}} = $html;

# HTML escape sub.  this is called-back by `parse_data' below in
# `html_preASP' to properly escape `<' and `&' characters coming from
# the SGML source.
$html_escapes{'&'} = '&amp;';
$html_escapes{'<'} = '&lt;';

$html_escape = sub {
    my ($data) = @_;

    # replace the char with it's HTML equivalent
    $data =~ s|([&<])|$html_escapes{$1}|ge;

    return ($data);
};

#
#  Translate character entities and escape HTML special chars.
#
sub html_preASP
{
  # note the conversion of `sdata_dirs' list to an anonymous array to
  # make a single argument
  $char_maps = load_char_maps ('.2html', [ Text::EntityMap::sdata_dirs() ]);

  open INFILE, "$TmpBase.1";
  open OUTFILE, ">$TmpBase.2";
  while (<INFILE>)
    {
      if (/^-/)
        {
	    my ($str) = $';
	    chop ($str);
	    print OUTFILE "-" . parse_data ($str, $char_maps, $html_escape) . "\n";
        }
      elsif (/^A/)
        {
	  /^A(\S+) (IMPLIED|CDATA|NOTATION|ENTITY|TOKEN)( (.*))?$/
	      || die "bad attribute data: $_\n";
	  my ($name,$type,$value) = ($1,$2,$4);
	  if ($type eq "CDATA")
	    {
	      # CDATA attributes get translated also
	      $value = parse_data ($value, $char_maps, $html_escape);
	    }
	  print OUTFILE "A$name $type $value\n";
        }
      else
        {
	  print OUTFILE $_;
        }
    }
  close INFILE;
  close OUTFILE;
}

#
#  Take the sgmlsasp output, and make something
#  useful from it.
#
sub html_postASP
{
  #
  #  Set various stuff as a result of option processing.
  #
  $ext   = "html";
  $ext   = "htm"  if $html->{dosnames};
  $lang  = "$LibDir/rep/html/$global->{language}";
  $img   = 0;
  $img   = 1 if $html->{imagebuttons};

  #
  # Bring in file
  #
  open FILE, "<$TmpBase.3" or die "Cannot open $TmpBase.3";
  @file = <FILE>;
  close FILE;

  #
  #  Find references
  #
  &{$fixref->{init}}($html->{'split'});
  LINE: foreach (@file) {
      foreach $pat (keys %{$fixref->{rules}}) {
          if (/$pat/) {
              # Call rule function then skip to next line
              &{$fixref->{rules}->{$pat}}; next LINE;
          }
      }
      &{$fixref->{defaultrule}};
  }
  &{$fixref->{finish}};

  #  
  #  Run through html2html, preserving stdout
  #  Also, handle prehtml.sed's tasks
  #
  open SAVEOUT, ">&STDOUT";
  open STDOUT, ">$FileName.$ext" or die qq(Cannot open "$FileName.$ext");

  &{$html2html->{init}}($html->{'split'}, $lang, $ext, $img, $FileName,
                        $fixref->{filenum}, $fixref->{lrec});
  LINE: foreach (@file) {
      s,<P></P>,,g; 			# remove empty <P></P> containers
      foreach $pat (keys %{$html2html->{rules}}) {
          if (/$pat/) {
              # Call rule function then skip to next line
              &{$html2html->{rules}->{$pat}}; next LINE;
          }
      }
      &{$html2html->{defaultrule}};
  }
  &{$html2html->{finish}};

  close STDOUT;
  open STDOUT, ">&SAVEOUT";

  return 0;
}

1;

