#
#  fmt_latex2e.pl
#
#  $Id: fmt_latex2e.pl,v 1.5 1997/05/21 10:06:29 cg Exp $
#
#  LaTeX-specific driver stuff
#
#   Copyright 1996, Cees de Groot
#

$latex2e = {};
$latex2e->{NAME} = "latex2e";
$latex2e->{HELP} = <<EOF;
  Note that this output format requires LaTeX 2e.

EOF
$latex2e->{OPTIONS} = [
   { option => "output", type => "l", 
     'values' => [ "dvi", "tex", "ps" ], short => "o" },
   { option => "bibtex", type => "f",  short => "b" },
   { option => "quick",  type => "f",  short => "q" }
];
$latex2e->{output} = "dvi";
$latex2e->{quick}  = 0;
$latex2e->{bibtex}  = 0;
$latex2e->{preNSGMLS} = << 'EOF';
  $NsgmlsOpts .= " -ifmttex ";
EOF
$latex2e->{preASP} = latex2e_preASP;
$latex2e->{postASP} = latex2e_postASP;

$Formats{$latex2e->{NAME}} = $latex2e;


# extra `\\' here for standard `nsgmls' output
$latex2e_escapes{'#'} = '\\\\#';
$latex2e_escapes{'$'} = '\\\\$';
$latex2e_escapes{'%'} = '\\\\%';
$latex2e_escapes{'&'} = '\\\\&';
$latex2e_escapes{'~'} = '\\\\~{}';
$latex2e_escapes{'_'} = '\\\\_';
$latex2e_escapes{'^'} = '\\\\^{}';
$latex2e_escapes{'\\'} = '\\verb+\\+';
$latex2e_escapes{'{'} = '\\\\{';
$latex2e_escapes{'}'} = '\\\\}';
$latex2e_escapes{'>'} = '{$>$}';
$latex2e_escapes{'<'} = '{$<$}';	# wouldn't happen, but that's what'd be
$latex2e_escapes{'|'} = '{$|$}';

# passed to `parse_data' below in latex2e_preASP
$latex2e_escape = sub {
    my ($data) = @_;

    if (!$in_verb) {
	# escape special characters
	$data =~ s|([#\$%&~_^\\{}<>\|])|$latex2e_escapes{$1}|ge;
    }

    return ($data);
};

#
#  Translate character entities and escape LaTeX special chars.
#
sub latex2e_preASP
{
  # note the conversion of `sdata_dirs' list to an anonymous array to
  # make a single argument
  $tex_char_maps = load_char_maps ('.2tex', [ Text::EntityMap::sdata_dirs() ]);

  # ASCII char maps are used in the verbatim environment because TeX
  # ignores all the escapes
  $ascii_char_maps = load_char_maps ('.2ab', [ Text::EntityMap::sdata_dirs() ]);

  $char_maps = $tex_char_maps;

  # used in `latex2e_escape' anonymous sub to switch between escaping
  # characters from SGML source or not, depending on whether we're in
  # a VERB or CODE environment or not
  $in_verb = 0;

  open INFILE, "$TmpBase.1";
  open OUTFILE, ">$TmpBase.2";
  while (<INFILE>)
    {
      if (/^-/)
        {
	    my ($str) = $';
	    chop ($str);
	    print OUTFILE "-" . parse_data ($str, $char_maps, $latex2e_escape) . "\n";
        }
      elsif (/^A/)
        {
	  /^A(\S+) (IMPLIED|CDATA|NOTATION|ENTITY|TOKEN)( (.*))?$/
	      || die "bad attribute data: $_\n";
	  my ($name,$type,$value) = ($1,$2,$4);
	  if ($type eq "CDATA")
	    {
	      # CDATA attributes get translated also
	      $value = parse_data ($value, $char_maps, $latex2e_escape);
	    }
	  print OUTFILE "A$name $type $value\n";
        }
      elsif (/^\((VERB|CODE)/)
        {
	  print OUTFILE $_;
	  # going into VERB/CODE section
	  $in_verb = 1;
	  $char_maps = $ascii_char_maps;
	}
      elsif (/^\)(VERB|CODE)/)
        {
	  print OUTFILE $_;
	  # leaving VERB/CODE section
	  $in_verb = 0;
	  $char_maps = $tex_char_maps;
	}
      else
        {
	  print OUTFILE $_;
        }
    }
  close INFILE;
  close OUTFILE;
}

#
#  Take the sgmlsasp output, and make something
#  useful from it.
#
sub latex2e_postASP
{
  $ENV{TEXINPUTS} .= ":$LibDir";

  #
  #  Set the correct \documentclass options. The if statement is just
  #  a small optimization.
  #
  if ($global->{language} ne "en" ||
      $global->{papersize} ne "a4")
    {
      $replace = '\documentclass[' . $global->{papersize} . 'paper,' . 
	Lang::ISO2English ($global->{language}) . ']';
      open LTXFILE, "$TmpBase.3";
      open OUTFILE, ">$FileName.tex";
      while (<LTXFILE>)
        {
	  s/\\documentclass\[a4paper,english\]/$replace/;
	  print OUTFILE;
	}
      close LTXFILE;
      close OUTFILE;
    }
  else
    {
      `mv $TmpBase.3 $FileName.tex`;
    }

  #
  #  LaTeX, dvips, and assorted cleanups.
  #
  system "latex $FileName.tex" || die "LaTeX problem";
  $latex2e->{bibtex} && system "bibtex $FileName";
  $latex2e->{quick} || system "latex $FileName.tex";
  $latex2e->{quick} || system "latex $FileName.tex";
  if ($global->{debug} == 0)
    {
      @suffixes = qw(log blg aux toc lof lot dlog bbl);
      for $suf (@suffixes)
        {
          unlink "$FileName.$suf";
        }
    }
  if ($latex2e->{output} eq "tex")
    { 
      unlink "$FileName.dvi"; 
      return 0; 
    }
  if ($latex2e->{output} eq "dvi")
    {
      $global->{debug} || unlink "$FileName.tex";
      return 0;
    }
  `dvips -q -t $global->{papersize} -o $FileName.ps $FileName.dvi`;
  $global->{debug} || `rm $FileName.dvi`;

  return 0;
}

1;
